<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:template match="/">
    <weight>
      <!-- Group orders by customer ID and compute average weight for each customer -->
      <xsl:variable name="customers">
        <xsl:for-each-group select="//Order" group-by="CustomerID">
          <customer id="{current-grouping-key()}" 
                weight="{avg(current-group()/ShipInfo/Freight)}" />
        </xsl:for-each-group>
      </xsl:variable>

      <xsl:call-template name="weightcategory">
        <xsl:with-param name="min" select="500" />
        <xsl:with-param name="name" select="'heavy'" />
        <xsl:with-param name="customers" select="$customers" />
      </xsl:call-template>
      <xsl:call-template name="weightcategory">
        <xsl:with-param name="min" select="10" />
        <xsl:with-param name="max" select="500" />
        <xsl:with-param name="name" select="'medium'" />
        <xsl:with-param name="customers" select="$customers" />
      </xsl:call-template>
      <xsl:call-template name="weightcategory">
        <xsl:with-param name="max" select="10" />
        <xsl:with-param name="name" select="'light'" />
        <xsl:with-param name="customers" select="$customers" />
      </xsl:call-template>
    </weight>
  </xsl:template>

  <xsl:template name="weightcategory">
    <xsl:param name="min" select="-1" />
    <xsl:param name="max" select="-1" />
    <xsl:param name="name" />
    <xsl:param name="customers" />

    <xsl:element name="{$name}">
      <xsl:for-each select="$customers/customer[
        ($max lt 0 or number(@weight) lt $max) and
        ($min lt 0 or number(@weight) ge $min)]">
        <customer id="{@id}"/>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>
</xsl:stylesheet>
